# -*- coding: utf-8 -*-
# Copyright 2020 ACSONE SA/NV (<http://acsone.eu>)
# License GPL-3.0 or later (http://www.gnu.org/licenses/gpl.html).

import os
import re
from typing import Iterable, Optional

import click

from .main import main
from .tools import check_output, find_python

_EQUAL_RE = re.compile(r"^([A-Za-z0-9-_.]+)\s*[@=]")
_EGG_RE = re.compile(r"egg=([A-Za-z0-9-_.]+)")


def _canonicalize(distribution: str) -> str:
    """Canonicalize a distribution name"""
    return distribution.lower().replace("_", "-")


def _req_dist(req: str) -> Optional[str]:
    """Find a distribution name in a pip requirement line"""
    mo = _EQUAL_RE.search(req) or _EGG_RE.search(req)
    if mo:
        return _canonicalize(mo.group(1))
    return None


def _list_depends(distribution: str, python: str) -> Iterable[str]:
    """Obtain transitive dependencies of a distributions"""
    _list_depends = os.path.join(os.path.dirname(__file__), "_list_depends")
    cmd = [find_python(python), _list_depends, distribution]
    dependencies = check_output(cmd).strip().split("\n")
    return {_canonicalize(d) for d in dependencies}


def _pip_freeze(python: str) -> Iterable[str]:
    cmd = [find_python(python), "-m", "pip", "freeze"]
    frozen = check_output(cmd).strip().split("\n")
    return frozen


def _freeze(distribution: str, python: str) -> Iterable[str]:
    # get dependencies
    dependencies = _list_depends(distribution, python)
    # call regular pip freeze
    frozen = _pip_freeze(python)
    # filter out pip freeze lines that are not in dependencies
    for req in frozen:
        req_dist = _req_dist(req)
        if req_dist and req_dist not in dependencies:
            continue
        yield req


@click.command()
@click.argument("distribution")
@click.option("--python", "-p", default="python")
def freeze(distribution: str, python: str) -> None:
    """pip freeze, but output only dependencies of a given distribution."""
    for req in _freeze(distribution, python):
        print(req)


main.add_command(freeze)



import textwrap

from click.testing import CliRunner

from acsoo.pr_status import pr_status


def test_pr_status_basic():
    runner = CliRunner()
    with runner.isolated_filesystem():
        with open("requirements.txt.in", "w") as f:
            f.write(
                textwrap.dedent(
                    """\
                    # closed not merged
                    -e git+https://github.com/OCA/mis-builder@refs/pull/298/head#\
                    subdirectory=setup/mis_builder&egg=odoo13-addon-mis_builder
                    # closed and merged
                    odoo10-addon-suspend_security @ \
                    git+https://github.com/OCA/server-tools@refs/pull/100/head#\
                    subdirectory=setup/suspend_security
                    # closed and merged
                    odoo14-addon-mis_builder @ \
                    git+https://github.com/acsone/mis-builder@14.0-350-port#\
                    subdirectory=setup/mis_builder \
                    # https://github.com/OCA/mis-builder/pull/359
                    """
                )
            )
        res = runner.invoke(pr_status, color=False)
        assert res.exit_code == 0
        assert "OCA/mis-builder/pull/298 is closed" in res.output
        assert "OCA/server-tools/pull/100 is merged" in res.output
        assert "OCA/mis-builder/pull/359 is merged" in res.output



# -*- coding: utf-8 -*-
# Copyright 2020 ACSONE SA/NV (<http://acsone.eu>)
# License GPL-3.0 or later (http://www.gnu.org/licenses/gpl.html).

import logging
import os
import re
from collections import namedtuple

import click
import httpx

from .main import main

_logger = logging.getLogger(__name__)


PR = namedtuple("PR", ["org", "repo", "pr"])
PR_RE = re.compile(
    r".*github.com.(?P<org>[a-zA-Z-_]+)/(?P<repo>[a-zA-Z-_]+)"
    r".*@refs/pull/(?P<pr>[0-9]+)/head"
)
PR_URL_RE = re.compile(
    r".*https://github.com/(?P<org>[^/]+)/(?P<repo>[^/]+)/pull/(?P<pr>[0-9]+).*"
)


def looks_like_req_file(filename):
    return ("requirements" in filename or "constraints" in filename) and (
        filename.endswith(".txt") or filename.endswith(".txt.in")
    )


def display_state(rjson):
    state = rjson["state"]
    merged = rjson["merged"]
    if state == "open":
        return click.style("open", fg="white")
    elif state == "closed" and merged:
        return click.style("merged", fg="magenta")
    elif state == "closed" and not merged:
        return click.style("closed", fg="red")
    else:
        return click.style(state, fg="yellow")


def search_prs():
    for reqfile in os.listdir("."):
        if not looks_like_req_file(reqfile):
            continue
        click.secho("Scanning " + reqfile, dim=True)
        with open(reqfile) as f:
            for line in f:
                mo = PR_RE.match(line) or PR_URL_RE.match(line)
                if not mo:
                    continue
                yield PR(**mo.groupdict())


@click.command()
def pr_status():
    """Show status of PR found in requirement files."""
    for pr in search_prs():
        r = httpx.get(f"https://api.github.com/repos/{pr.org}/{pr.repo}/pulls/{pr.pr}")
        r.raise_for_status()
        state = display_state(r.json())
        click.echo(f"https://github.com/{pr.org}/{pr.repo}/pull/{pr.pr} is {state}")


main.add_command(pr_status)
